#!/usr/bin/perl --
# Script to transform tab delimited address data into LDAP.



use Net::LDAP;
use strict;

my $infile = $ARGV[0] || "addresses.txt";

my $conn = Net::LDAP->new("ldap.daisypark.net") or # Replace with your LDAP server
  die "ERROR: Can't connect: $@";

# make a authenticated connection
$conn->bind( dn => 'cn=Manager, dc=daisypark, dc=net',
            password => 'secret',
          );

my $progress = 1;
$|++;
while ( my $rec = get_next_record($infile) ) {

    next unless "$rec->{FirstName}$rec->{LastName}";

    my $result = $conn->add(
      dn => "cn=$rec->{FirstName} $rec->{LastName}, dc=daisypark, dc=net",

      attr => [
           cn	=> "$rec->{FirstName} $rec->{LastName}",

           sn	=> $rec->{LastName},
           mail	=> $rec->{EmailAddress},
           telephoneNumber	=> $rec->{HomePhone},
           street	=> $rec->{Address},
           postalCode	=> $rec->{PostalCode},
           st	=> $rec->{StateOrProvince},
           l	=> $rec->{StateOrProvince},
           c	=> 'US',
           objectclass	=> [ 'top', 'person', 
                                'organizationalPerson', 'inetOrgPerson',
                              ],
             ],
    );

    if ( $result->code ) {
        warn "WARN: Failed to add entry: $rec->{FirstName}, $rec->{LastName}: ",
        sprintf "%x", $result->code;
    }

    printf "seen: %d\r", $progress++;
}

print "\nClosing LDAP connection\n";
$conn->unbind;
print "done\n";

#------
# Subroutines
#------
# get_next_record() opens the tab-delimited file, returning successive
# hashref records, one per entry.
{
  my ($seen, @headers); # persistent variables
  sub get_next_record {
      my $file = shift || return;

      unless ( $seen ) {
          open F, $file or die "ERROR: Can't open $file: $!";
          @headers   = split "\t", scalar ;
          $seen = 1;
      }

      my $line;
      unless ( defined ($line = <F>) ) {
          close F;
          return;
      }

      until ( $line ) {
          chomp $line;
          $line =~ s/\s*$//;
      }

      my $record = {};

      @$record{ @headers } = (split "\t", $line);

      return $record;
  }
}
